__version__ = "0.3.0"

from .Content import Content # noqa: F401
from .Text import Text # noqa: F401
from .Post import Post # noqa: F401
from .Comment import Comment # noqa: F401
from .Proxy import Proxy # noqa: F401

from .version_check import check_new_version # noqa: F401
check_new_version()


#%%
from wtti import Text
import uuid
from random import randint
from time import sleep
import pandas as pd
from datetime import datetime
import dateutil
tz = dateutil.tz.gettz('Asia/Taipei')

#%%
def test_text_content():
    text = Text(text="Hello world!")
    assert text.text == "Hello world!"
    text.text = "Hello pytest!"
    assert text.text == "Hello pytest!"

def test_author():
    text = Text(author="John Doe")
    assert text.author == "John Doe"
    text.author = "Jane Smith"
    assert text.author == "Jane Smith"

def test_published_time():
    # 測試輸入 datetime 物件
    time = datetime(2022, 1, 1, 12, 0, 0)
    text = Text(published_time=time)
    assert text.published_time == time.timestamp()
    new_time = datetime(2023, 3, 28, 12, 0, 0)
    text.published_time = new_time.timestamp()
    assert text.published_time == new_time.timestamp()
    # 測試輸入 timestamp
    time = datetime(2022, 1, 1, 12, 0, 0).timestamp()
    text = Text(published_time=time)
    assert text.published_time == time
    # 測試輸入字串
    time = "2023-03-28 12:00:00"
    text = Text(published_time=time)
    assert text.published_time == datetime.strptime(time, "%Y-%m-%d %H:%M:%S").replace(tzinfo=tz).timestamp()

def test_likes():
    text = Text(likes=10)
    assert text.likes == 10
    text.likes = 20
    assert text.likes == 20

def test_platform():
    text = Text(platform="Twitter")
    assert text.platform == "Twitter"
    text.platform = "Facebook"
    assert text.platform == "Facebook"

def test_url():
    text = Text(url="https://example.com")
    assert text.url == "https://example.com"
    
#%%
datetime(2022, 1, 1, 12, 0, 0).timestamp()


from .Text import Text
from .Comment import Comment

class Post(Text):
    """
    Post 為文章儲存單位的實體類別, 定義了一個文章的基本屬性
    
    Attributes:
        text_content: 內容的文字內容
        author: 內容的作者
        published_timestamp: 內容的發布時間
        likes: 內容的讚數
        platform: 內容所在的平台
        url: 內容所在的網址
        title: 內容的標題
        category: 內容的分類
        comments: 內容的留言列表，非直接記錄，而是透過 PostComment 類別記錄
        
    Methods:
        to_series: 將內容物轉換成 pandas.Series
        add_comments: 將留言加入內容物的留言列表
    """
    
    def __init__(self, text="", url="", author="", platform="", likes=-1, published_time=None, title="", category=""):
        super().__init__(text=text, url=url, author=author, platform=platform, likes=likes, published_time=published_time)
        self.comments_count = 0 # 留言數量
        self.comments = [] # 留言列表
        self.comments_uuid = [] # 留言的 uuid 列表
        self.data.update({
            "title": title,
            "category": category,
        })
        
    
    def add_comment(self, comment_text="", author="", likes=-1, published_time=None):
        """
        將留言加入內容物的留言列表
        
        Args:
            comment: 要加入的留言
        """
        # 建立一個 Comment 物件
        comment = Comment(
            text=comment_text, author=author, likes=likes, published_time=published_time, 
            url=self.url, platform=self.platform, post_id=self.uuid,
            )
        # 將該物件加入內容物的留言列表
        self.comments.append(comment)
        # 留言數量 + 1
        self.comments_count += 1
        # 將該留言的 uuid 加入內容物的留言 uuid 列表
        self.comments_uuid.append(comment.uuid)
        return comment.uuid
    



#%%
import requests
from wtti import __version__ as VERSION
import re

NEW_VERSION_TIP = """
──────────────────────────────────────────────────────
New version available \033[31m{version}\033[0m → \033[32m{new_version}\033[0m
Run \033[33mpip install --upgrade MTTI\033[0m to update!
"""

#%%
def check_new_version():
    try:
        url = "https://pypi.org/simple/wtti/"
        resp = requests.get(url, timeout=3)
        html = resp.text

        last_stable_version = re.findall(r"WTTI-([\d.]*?).tar.gz", html)[-1]
        now_version = VERSION


        if now_version < last_stable_version:
            new_version = f"MTTI=={last_stable_version}"
            if new_version:
                version = f"MTTI=={VERSION.replace('-beta', 'b')}"
                tip = NEW_VERSION_TIP.format(version=version, new_version=new_version)
                print(tip)
    except Exception as e:
        pass



import pandas as pd
import uuid
from time import time

class Content:
    """
    Content 為一個抽象類別, 用來定義一個內容物的基本屬性
    並提供一個 to_series 方法, 用來轉換成 pandas.Series

    Attributes:
        uuid: 內容物的 uuid
        created_timestamp: 內容物的建立時間
        modified_timestamp: 內容物的最後修改時間
        data: 內容物的資料
    
    Methods:
        to_series: 將內容物轉換成 pandas.Series
        
    """
    
    def __init__(self):
        self.uuid = uuid.uuid4()
        self.created_timestamp = int(time())
        self.modified_timestamp = self.created_timestamp
        self.data = {
            "uuid": self.uuid,
            "created_timestamp": self.created_timestamp,
            "modified_timestamp": self.modified_timestamp,
        }

    def __getitem__(self, key):
        return self.data[key]

    def __setitem__(self, key, value):
        # 檢查是否為內建屬性
        if key in ["uuid", "created_timestamp", "modified_timestamp"]:
            raise KeyError(f"Can not modify {key}")
        
        # 檢查要修改的屬性是無變化
        if key in self.data and self.data[key] == value:
            return
        # 更新 modified_timestamp
        else:
            self.modified_timestamp = int(time())
            self.data["modified_timestamp"] = self.modified_timestamp
            # 更新屬性
            self.data[key] = value

    def to_series(self):
        return pd.Series(self.data)



import requests
import re
import random

class Proxy:
    def __init__(self, headers=None, cookies={}, proxy_mode="random"):
        
        assert proxy_mode in ["random", "order", "score"], f"Invalid proxy_mode: {proxy_mode}, must be 'random', 'order', or 'score'."
        
        self.headers = headers if headers else {
            'User-Agent': "Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/111.0.0.0 Safari/537.36",        
        }
        self.cookies = cookies
        self.proxy_ips = self.get_proxy_ips()
        self.proxy_mode = proxy_mode
        
        if proxy_mode == "score":
            self.proxy_scores = {proxy_ip: 10 for proxy_ip in self.proxy_ips}
        self.success = {proxy_ip: 0 for proxy_ip in self.proxy_ips}
        self.failure = {proxy_ip: 0 for proxy_ip in self.proxy_ips}

    def get_proxy_ips(self):
        """從sslproxies.org網站取得代理IP列表"""
        response = requests.get("https://www.sslproxies.org/")
        assert response.status_code == 200, "Fails to get proxy IPs."
        proxy_ips = re.findall('\d+\.\d+\.\d+\.\d+:\d+', response.text)  #「\d+」代表數字一個位數以上
        return proxy_ips
    
    def request(self, 
                url, 
                method='GET', 
                data={},
                max_retries_per_proxy=3, 
                max_proxies_to_try=3,
                ignore_failure=True,
                use_local=False,
    ):
        """使用代理IP發送請求"""

        for proxy_ip in self.get_proxies(max_proxies_to_try):
            print(f"Trying proxy: {proxy_ip}")
            for i in range(max_retries_per_proxy):
                print(f"Retry {i+1}...", end="\r")
                try:
                    response = requests.request(
                        method=method, 
                        url=url, 
                        data=data,
                        headers=self.headers, 
                        cookies=self.cookies, 
                        proxies={'https': proxy_ip, 'http': proxy_ip}
                    )
                    response.raise_for_status()
                    self._record_success(proxy_ip)
                    return response
                except Exception as e:
                    self._record_failure(proxy_ip)
        
        if not ignore_failure:
            raise Exception("All proxies failed.")
        
        if use_local:
            print("All proxies failed, using local IP.")
            return requests.request(
                method=method, 
                url=url, 
                data=data,
                headers=self.headers, 
                cookies=self.cookies, 
            )
    
    def get_proxies(self, n=1):
        """取得要嘗試的代理IP
        Args:
            n (int): 要嘗試的代理IP數量
        Returns:
            proxies (list): 要嘗試的代理IP列表
        """
        n = min(n, len(self.proxy_ips))
        
        if self.proxy_mode == "random":
            proxies =  random.sample(self.proxy_ips, n)
        if self.proxy_mode == "order":
            proxies = self.proxy_ips[:n]
            self.proxy_ips = self.proxy_ips[n:] + self.proxy_ips[:n]
        if self.proxy_mode == "score":
            proxies = sorted(self.proxy_ips, key=lambda x: self.proxy_scores[x], reverse=True)[:n]
                
        return proxies
    
    def _record_success(self, proxy_ip):
        """記錄成功的代理IP"""
        self.success[proxy_ip] += 1
        if self.proxy_mode == "score":
            self.proxy_scores[proxy_ip] += 1
        
    def _record_failure(self, proxy_ip):
        """記錄失敗的代理IP"""
        self.failure[proxy_ip] += 1
        if self.proxy_mode == "score":
            self.proxy_scores[proxy_ip] -= 3
            
    def get_record(self):
        """取得代理IP的成功與失敗次數
        合併到一個dict
        """
        return {proxy_ip: {"success": self.success[proxy_ip], "failure": self.failure[proxy_ip]} for proxy_ip in self.proxy_ips}


#%%
from wtti.output import posts_to_dataframe
from wtti import Post
import pandas as pd

#%%
def test_posts_to_dataframe():
    
    P1 = Post(
    text="文章內容1",
    url="https://www.google.com",
    author="作者1",
    platform="平台1",
    likes=10,
    published_time="2020-01-01 00:00:00",
    title="文章標題1",
    category="OO板",
    )
    P2 = Post(
        text="文章內容2",
        url="https://www.google.com",
        author="作者2",
        platform="平台1",
        likes=5,
        published_time="2020-02-01 00:00:00",
        title="文章標題2",
        category="OO板",
    )
    P1.add_comment("留言內容1", "留言者1", 1, "2020-01-01 00:00:00")
    P1.add_comment("留言內容2", "留言者2", 2, "2020-01-02 00:00:00")
    
    posts_df, comments_df = posts_to_dataframe([P1, P2], merge=False)
    assert isinstance(posts_df, pd.DataFrame)
    assert len(posts_df) == 2
    assert list(posts_df["author"]) == ['作者1', '作者2']
    assert list(posts_df["title"]) == ['文章標題1', '文章標題2']
    assert list(posts_df["category"]) == ['OO板', 'OO板']
    assert list(posts_df["url"]) == ['https://www.google.com', 'https://www.google.com']
    assert list(posts_df["likes"]) == [10, 5]
    assert list(posts_df["published_timestamp"]) == [1577808000, 1580486400]
    
    assert isinstance(comments_df, pd.DataFrame)
    assert len(comments_df) == 2
    assert list(comments_df["author"]) == ['留言者1', '留言者2']
    assert list(comments_df["post_id"]) == [P1.uuid, P1.uuid]
    assert list(comments_df["likes"]) == [1, 2]
    assert list(comments_df["published_timestamp"]) == [1577808000, 1577894400]
    
    posts_df, comments_df = posts_to_dataframe([P1, P2], merge=True)
    assert isinstance(posts_df, pd.DataFrame)
    assert len(posts_df) == 4
    assert list(posts_df["author"]) == ['作者1', '留言者1', '留言者2', '作者2']
    assert list(posts_df["title"]) == ['文章標題1', '文章標題1', '文章標題1', '文章標題2']
    assert list(posts_df["category"]) == ['OO板', 'OO板', 'OO板', 'OO板']
    assert list(posts_df["url"]) == ['https://www.google.com', 'https://www.google.com', 'https://www.google.com', 'https://www.google.com']
    assert list(posts_df["likes"]) == [10, 1, 2, 5]
    assert list(posts_df["published_timestamp"]) == [1577808000, 1577808000, 1577894400, 1580486400]
    assert list(posts_df["content_type"]) == ['post', 'comment', 'comment', 'post']
    assert len(posts_df.iloc[0]["comments_uuid"]) == 2


#%%
from wtti.utils import convert_to_timestamp



#%%
def test_convert_to_timestamp():
    # 測試常見的日期時間格式的輸入
    assert convert_to_timestamp("2022-03-27 12:34:56") == 1648355696

    # 測試timestamp格式的輸入
    assert convert_to_timestamp(1648427696) == 1648427696

    # 測試無效格式的輸入
    try:
        convert_to_timestamp("invalid format")
        assert False, "Should raise ValueError for invalid format"
    except ValueError:
        pass
    



from .Text import Text

class Comment(Text):
    """
    Comment 為留言儲存單位的實體類別, 定義了一個留言的基本屬性
    
    Attributes:
        text_content: 內容的文字內容
        author: 內容的作者
        published_timestamp: 內容的發布時間
        likes: 內容的讚數
        platform: 內容所在的平台
        url: 內容所在的網址
        ---
        post_id: 內容所在的文章的 id
        comments: 留言的留言列表， # TODO: 未來可能會有，目前先保留
        
    Methods:
        to_series: 將內容物轉換成 pandas.Series
        add_comments: 將留言加入內容物的留言列表
    """
    
    def __init__(self, text="", url="", author="", platform="", likes=-1, published_time=None, post_id=None):
        super().__init__(text=text, url=url, author=author, platform=platform, likes=likes, published_time=published_time)
        self.post_id = post_id
        self.data.update({
            "post_id": post_id,
        })
        
    


from .Post import Post
import pandas as pd

def posts_to_dataframe(posts, merge=False):
    """
    將一個或多個 Post 物件轉換為 pandas DataFrame 格式。

    Args:
        posts (list or Post): 一個 Post 物件或多個 Post 物件的列表。
        merge (bool): 是否將 post 與其 comments 合併為一個 DataFrame。

    Returns:
        tuple: 如果 merge 為 True，回傳一個包含 posts 與 comments 的 DataFrame。
               如果 merge 為 False，回傳一個包含 posts 的 DataFrame 和一個包含 comments 的 DataFrame。
    """
    
    if not isinstance(posts, list):
        posts = [posts]
        
    if merge:
        post_data = []
        for post in posts:
            # 取出 post 的基本屬性，並將留言數量加入 post_data
            post_dict = post.data
            post_dict["content_type"] = "post"
            post_dict["comments_count"] = post.comments_count
            post_dict["comments_uuid"] = post.comments_uuid
            post_data.append(post_dict)
            # 取出 post 的每一個 comment，並將其轉換為一個字典
            for comment in post.comments:
                comment_dict = comment.data
                # 將 comment 所屬的 post 的 uuid 加入 comment_data
                comment_dict["post_uuid"] = post.uuid
                comment_dict["content_type"] = "comment"
                # 補上 post 的基本屬性
                comment_dict["title"] = post.data["title"]
                comment_dict["category"] = post.data["category"]
                post_data.append(comment_dict)
        # 將 post_data 和 comment_data 分別轉換為 DataFrame
        posts_df = pd.DataFrame(post_data)
        comments_df = None
    else:
        post_data = []
        comment_data = []
        for post in posts:
            # 取出 post 的基本屬性，並將留言數量加入 post_data
            post_dict = post.data
            post_dict["comments_count"] = post.comments_count
            post_data.append(post_dict)
            # 取出 post 的每一個 comment，並將其轉換為一個字典
            for comment in post.comments:
                comment_dict = comment.data
                # 將 comment 所屬的 post 的 uuid 加入 comment_data
                comment_data.append(comment_dict)
        # 將 post_data 和 comment_data 分別轉換為 DataFrame
        posts_df = pd.DataFrame(post_data)
        comments_df = pd.DataFrame(comment_data)
    return posts_df, comments_df


#%%
from datetime import datetime
from wtti import Post

#%%
def test_add_comment():
    post = Post(
        text="This is a test post",
        url="https://example.com/test",
        author="Test author",
        platform="Test platform",
        likes=10,
        published_time=datetime.now(),
        title="Test title",
        category="Test category",
    )
    comment_text = "This is a test comment"
    author = "Test commenter"
    likes = 5
    published_time = datetime.now()
    comment_uuid = post.add_comment(comment_text=comment_text, author=author, likes=likes, published_time=published_time)

    assert len(post.comments) == 1
    assert post.comments_count == 1
    assert post.comments[0].text == comment_text
    assert post.comments[0].author == author
    assert post.comments[0].likes == likes
    # assert post.comments[0].published_time == published_time
    assert post.comments[0].url == post.url
    assert post.comments[0].platform == post.platform
    assert post.comments[0].post_id == post.uuid
    assert post.comments[0].uuid == comment_uuid



#%%
from wtti import Proxy
import pytest
from unittest import mock
from requests.exceptions import HTTPError

#%%
# 測試代理IP是否可以正常取得
def test_get_proxy_ips():
    proxy = Proxy()
    assert len(proxy.get_proxy_ips()) > 0

# 測試代理IP是否可以正常發送請求
def test_request():
    proxy = Proxy()
    response = proxy.request("https://www.example.com")
    assert response.status_code == 200


from datetime import datetime
import dateutil
tz = dateutil.tz.gettz('Asia/Taipei')


def convert_to_timestamp(time_input):
    """
        自動判斷輸入的時間格式，並轉換成timestamp
        支援的時間格式有：
            1. timestamp
            2. "%Y-%m-%d %H:%M:%S"
            3. datetime.datetime
    """
    # datetime格式
    if isinstance(time_input, datetime):
        # 考慮到有可能輸入的datetime沒有設定時區，所以這邊強制設定成台北時區
        time_input = time_input.astimezone(tz)
        return int(time_input.timestamp())
    try:
        # timestamp格式
        timestamp = datetime.fromtimestamp(int(time_input), tz=tz)
    except (ValueError, TypeError):
        try:
            # 常見的日期時間格式
            timestamp = datetime.strptime(time_input, "%Y-%m-%d %H:%M:%S").replace(tzinfo=tz)
            timestamp = timestamp.astimezone(tz)
            
        except (ValueError, TypeError):
            # 如果都不是上述格式，拋出例外
            raise ValueError("Invalid time format, please use timestamp or '%Y-%m-%d %H:%M:%S' format.")
    return int(timestamp.timestamp())



import setuptools
from wtti import __version__

with open("README.md", "r", encoding="utf-8") as fh:
    long_description = fh.read()

setuptools.setup(
    name="WTTI",
    version=__version__,
    author="owo",
    author_email="contact@owomail.cc",
    description="Webpage Text Transformation Interface，將網頁文章內容以結構化方式儲存，以方便後續分析應用。",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/Keycatowo/WTTI",
    packages=setuptools.find_packages(),
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
    install_requires=[
        "pandas",
    ],
)


#%%
from .Content import Content
from .utils import convert_to_timestamp
import pandas as pd


class Text(Content):
    """
    Text 為一個內容物的抽象類別, 定義了一個內容物的基本屬性

    Attributes:
        text_content: 內容的文字內容
        author: 內容的作者
        published_timestamp: 內容的發布時間
        likes: 內容的讚數
        platform: 內容所在的平台
        url: 內容所在的網址
    
    Methods:
        to_series: 將內容物轉換成 pandas.Series
    """
    
    def __init__(self, text="", url="", author="", platform="", likes=None, published_time=None):
        super().__init__()
        self.data.update({
            "text_content": text,
            "author": author,
            "published_timestamp": convert_to_timestamp(published_time) if published_time else None,
            "likes": likes,
            "platform": platform,
            "url": url,
        })
        
    def __repr__(self) -> str:
        # 建立一個字串，用來儲存要回傳的內容
        return_str = ""
        # 將所有的data都列出來，不包含text_content
        for key, value in self.data.items():
            if key != "text_content":
                return_str += f"{key}: {value} \n"
        return_str += "-"*20
        # 將text_content的內容列出來
        return_str += f"\n{self.data['text_content']}"
        return_str += "\n"+ "="*20 + "\n\n"
        return return_str
        
    def __str__(self) -> str:
        return self.data["text_content"]
        
    @property
    def text(self):
        return self.data["text_content"]
    
    @text.setter
    def text(self, value):
        super().__setitem__("text_content", value)
        self["text_content"] = value

    @property
    def author(self):
        return self.data["author"]
    
    @author.setter
    def author(self, value):
        super().__setitem__("author", value)
        self["author"] = value
    
    @property
    def published_time(self):
        return self.data["published_timestamp"]
    
    @published_time.setter
    def published_time(self, value):
        super().__setitem__("published_timestamp", value)
        self["published_timestamp"] = value
    
    @property
    def likes(self):
        return self.data["likes"]
    
    @likes.setter
    def likes(self, value):
        super().__setitem__("likes", value)
        self["likes"] = value
    
    @property
    def platform(self):
        return self.data["platform"]
    
    @platform.setter
    def platform(self, value):
        super().__setitem__("platform", value)
        self["platform"] = value
    
    @property
    def url(self):
        return self.data["url"]
    
    @url.setter
    def url(self, value):
        self["url"] = value

    



#%%
from wtti import Content
import uuid
from time import time
from random import randint
from time import sleep
import pandas as pd

def test_Content():
    # 建立一個 Content 物件
    created_time = int(time())
    content = Content()
    # 檢查 uuid 是否為 uuid.UUID 物件
    assert isinstance(content["uuid"], uuid.UUID)
    # 檢查 timestamp 是否為 int
    assert isinstance(content["created_timestamp"], int)
    assert isinstance(content["modified_timestamp"], int)
    # 檢查 timestamp 是否為當前時間, 以 5 秒為容許誤差
    assert abs(content["created_timestamp"] - created_time) <= 5
    assert abs(content["modified_timestamp"] - created_time) <= 5
    # 檢查是否可以修改屬性，用隨機數模擬
    sleep(2)
    for _ in range(10):
        key = str(uuid.uuid4())
        value = randint(0, 100)
        content[key] = value
        assert content[key] == value
    # 此時 timestamp 應該要更新
    assert content["modified_timestamp"] > created_time 
    # 檢查若要修改內建屬性會拋出錯誤
    try:
        content["uuid"] = "123"
    except KeyError as e:
        assert str(e) == "'Can not modify uuid'" or str(e) == "Can not modify uuid"
    # 檢查若要修改的屬性值無變化，不會更新 timestamp
    content["test"] = 123
    modified_time = content["modified_timestamp"]
    sleep(2)
    content["test"] = 123
    assert content["modified_timestamp"] == modified_time
    # 檢查 to_series() 是否能正常轉換
    series = content.to_series()
    assert isinstance(series, pd.Series)
    assert series["uuid"] == content["uuid"]
    assert series["created_timestamp"] == content["created_timestamp"]
    assert series["modified_timestamp"] == content["modified_timestamp"]
    